@@ -11,20 +11,16 @@
 # ABI_MIN is the minimum postgres version required when the extension was build against LATEST
 #
 
-PG14_EARLIEST = "14.0"
-PG14_LATEST = "14.17"
-PG14_ABI_MIN = "14.0"
+PG15_EARLIEST = "15.10"
+PG15_LATEST = "15.13"
+PG15_ABI_MIN = "15.10"
 
-PG15_EARLIEST = "15.0"
-PG15_LATEST = "15.12"
-PG15_ABI_MIN = "15.0"
+PG16_EARLIEST = "16.6"
+PG16_LATEST = "16.9"
+PG16_ABI_MIN = "16.6"
 
-PG16_EARLIEST = "16.0"
-PG16_LATEST = "16.8"
-PG16_ABI_MIN = "16.0"
+PG17_EARLIEST = "17.2"
+PG17_LATEST = "17.5"
+PG17_ABI_MIN = "17.2"
 
-PG17_EARLIEST = "17.0"
-PG17_LATEST = "17.4"
-PG17_ABI_MIN = "17.0"
-
-PG_LATEST = [PG14_LATEST, PG15_LATEST, PG16_LATEST, PG17_LATEST]
+PG_LATEST = [PG15_LATEST, PG16_LATEST, PG17_LATEST]
