@@ -51,6 +51,7 @@ default_ignored_tests = {
     "bgw_launcher",
     "telemetry",
     "memoize",
+    "net",
 }
 
 # Tests that we do not run as part of a Flake tests
@@ -156,8 +157,8 @@ def macos_config(overrides):
             "extra_packages": "",
             "ignored_tests": default_ignored_tests.union(macos_ignored_tests),
             "os": "macos-13",
-            "pg_extra_args": "--with-libraries=/usr/local/opt/openssl@3/lib --with-includes=/usr/local/opt/openssl@3/include --without-icu",
-            "pg_extensions": "postgres_fdw test_decoding pageinspect pgstattuple",
+            "pg_extra_args": "--enable-debug --with-libraries=/usr/local/opt/openssl@3/lib --with-includes=/usr/local/opt/openssl@3/include --without-icu",
+            "pg_extensions": "postgres_fdw test_decoding",
             "pginstallcheck": True,
             "tsdb_build_args": "-DASSERTIONS=ON -DREQUIRE_ALL_TESTS=ON -DOPENSSL_ROOT_DIR=/usr/local/opt/openssl@3",
         }
