@@ -13,6 +13,8 @@
 #include <nodes/parsenodes.h>
 #include <storage/lockdefs.h>
 
+#include "chunk_tuple_routing.h"
+
 typedef struct ChunkDispatch ChunkDispatch;
 typedef struct CopyChunkState CopyChunkState;
 typedef struct Hypertable Hypertable;
@@ -25,6 +27,7 @@ typedef struct CopyChunkState
 	Relation rel;
 	EState *estate;
 	ChunkDispatch *dispatch;
+	ChunkTupleRouting *ctr;
 	CopyFromFunc next_copy_from;
 	CopyFromState cstate;
 	TableScanDesc scandesc;
