@@ -32,9 +32,9 @@
 
 #include <math.h>
 
-#include "compat/compat.h"
 #include "allpaths.h"
 #include "chunk.h"
+#include "cross_module_fn.h"
 #include "planner/planner.h"
 
 static void set_rel_pathlist(PlannerInfo *root, RelOptInfo *rel, Index rti, RangeTblEntry *rte);
@@ -152,7 +152,7 @@ ts_set_append_rel_pathlist(PlannerInfo *root, RelOptInfo *parent_rel, Index pare
 		AppendRelInfo *appinfo = (AppendRelInfo *) lfirst(l);
 
 		/* append_rel_list contains all append rels; ignore others */
-		if (appinfo->parent_relid != (Index) parent_rt_index)
+		if (appinfo->parent_relid != parent_rt_index)
 			continue;
 
 		/* Re-locate the child RTE and RelOptInfo */
@@ -195,7 +195,8 @@ ts_set_append_rel_pathlist(PlannerInfo *root, RelOptInfo *parent_rel, Index pare
 			 */
 			Assert(chunk != NULL);
 
-			if (!ts_chunk_is_partial(chunk) && ts_chunk_is_compressed(chunk))
+			if (!ts_chunk_is_partial(chunk) && ts_chunk_is_compressed(chunk) &&
+				!ts_is_hypercore_am(chunk->amoid))
 			{
 				child_rel->indexlist = NIL;
 			}
@@ -217,6 +218,11 @@ ts_set_append_rel_pathlist(PlannerInfo *root, RelOptInfo *parent_rel, Index pare
 		 * Child is live, so add it to the live_childrels list for use below.
 		 */
 		live_childrels = lappend(live_childrels, child_rel);
+
+		/* If consider startup costs on chunks because we can apply SkipScan, should also consider
+		 * startup costs on a hypertable */
+		if (child_rel->consider_startup)
+			parent_rel->consider_startup = true;
 	}
 
 	/* Add paths to the append relation. */
