@@ -17,4 +17,4 @@
 extern void ts_trigger_create_on_chunk(Oid trigger_oid, const char *chunk_schema_name,
 									   const char *chunk_table_name);
 extern TSDLLEXPORT void ts_trigger_create_all_on_chunk(const Chunk *chunk);
-extern bool ts_relation_has_transition_table_trigger(Oid relid);
+extern void ts_check_unsupported_triggers(Oid relid);
