@@ -8,7 +8,6 @@
 #include <postgres.h>
 #include <catalog/pg_type.h>
 
-#include "hypertable.h"
 #include "ts_catalog/catalog.h"
 
 typedef struct CompressionSettings
@@ -16,18 +15,21 @@ typedef struct CompressionSettings
 	FormData_compression_settings fd;
 } CompressionSettings;
 
-TSDLLEXPORT CompressionSettings *ts_compression_settings_create(Oid relid, ArrayType *segmentby,
+TSDLLEXPORT CompressionSettings *ts_compression_settings_create(Oid relid, Oid compress_relid,
+																ArrayType *segmentby,
 																ArrayType *orderby,
 																ArrayType *orderby_desc,
 																ArrayType *orderby_nullsfirst);
 TSDLLEXPORT CompressionSettings *ts_compression_settings_get(Oid relid);
-TSDLLEXPORT CompressionSettings *ts_compression_settings_materialize(Oid ht_relid, Oid dst_relid);
+TSDLLEXPORT CompressionSettings *ts_compression_settings_get_by_compress_relid(Oid relid);
+TSDLLEXPORT CompressionSettings *ts_compression_settings_materialize(const CompressionSettings *src,
+																	 Oid relid, Oid compress_relid);
 TSDLLEXPORT bool ts_compression_settings_delete(Oid relid);
+TSDLLEXPORT bool ts_compression_settings_delete_by_compress_relid(Oid relid);
+TSDLLEXPORT bool ts_compression_settings_delete_any(Oid relid);
 TSDLLEXPORT bool ts_compression_settings_equal(const CompressionSettings *left,
 											   const CompressionSettings *right);
 
 TSDLLEXPORT int ts_compression_settings_update(CompressionSettings *settings);
-
-TSDLLEXPORT void ts_compression_settings_rename_column(Oid relid, char *old, char *new);
-TSDLLEXPORT void ts_compression_settings_rename_column_hypertable(Hypertable *ht, char *old,
-																  char *new);
+TSDLLEXPORT void ts_compression_settings_rename_column_cascade(Oid parent_relid, const char *old,
+															   const char *new);
